%option yylineno
%{

/* include our config header.  nothing else should be #included here */
#include "config.h"

/* parser global variables required */
char *parser_pos1, *parser_pos2;
char *parser_name = NULL;
char *parser_pmesg = NULL;
char *parser_ip = NULL;
char *parser_root = NULL;
char *parser_type = NULL;
int  parser_i_type = 0;
char *parser_port;
int  parser_i_port = 0;
char *parser_numfailures = NULL;
int parser_i_numfailures;
char *parser_desc = NULL;
char *parser_spawn = NULL;
char *parser_contact = NULL;
char *parser_child = NULL;
char parser_reverse = 0;
char *parser_sender = NULL;
char *parser_subject = NULL;
char *parser_upcolor = NULL;
char *parser_downcolor = NULL;
char *parser_recentcolor = NULL;
char *parser_replyto = NULL;
char *parser_errorsto = NULL;
char *parser_header = NULL;
char *parser_authkey = NULL;
char *parser_statusfile = NULL;
char *parser_logging = NULL;
int parser_logging_fac;
int parser_statusfile_type;
struct nei_list *parser_dep = NULL;
struct nei_list *parser_dep_tmp = NULL;   /* Used when adding new elements */
char *parser_page = NULL;
char *parser_also = NULL;
char *parser_secret = NULL;
char *parser_username = NULL;
char *parser_password = NULL;
char *parser_url = NULL;
char *parser_urltext = NULL;
char *parser_include = NULL;
char *parser_value = NULL;
char *parser_eq = NULL;
int parser_i_queuetime;
char *parser_queuetime = NULL;
int parser_i_dnsexpire;
char *parser_dnsexpire = NULL;
int parser_i_dnslog;
char *parser_dnslog = NULL;
int parser_i_pageinterval;
char *parser_pageinterval = NULL;
int parser_i_maxqueued;
char *parser_maxqueued = NULL;
int parser_showupalso; /*  */
int parser_nologconnects;
int parser_noheartbeat;
int parser_nosubject;
int temp_int;
char *temp_char;

/* end parser global variables */

/* Function definitions that are found later in the program */
void add_object();
void open_new_file(char *, FILE *);
void free_struct_nei_list(struct nei_list *);


/* end function predefs */

/* external variables not found in config.h */

extern struct all_elements_list *parser_head;

/* end external declarations not found in config.h */


%}

WS	[\t ]

%%

([;#])+.*\n	/* Ignore comments */

[\t ]+       /* ignore whitespace */

\n           /* ignore newlines */


set[\t ]+[^;]+;       {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_eq != NULL)
  {
        FREE(parser_eq);
  }
  parser_eq = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_eq");
  strncpy( parser_eq, parser_pos1, parser_pos2 - parser_pos1 );

  parser_eq[ parser_pos2 - parser_pos1 ] = '\0';


  parser_pos1 = index( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ' ' );
  if (parser_value != NULL)
  {
        FREE(parser_value);
  }
  parser_value = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_value");
  strncpy( parser_value, parser_pos1, parser_pos2 - parser_pos1 );

  parser_value[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug) 
    print_err(0, "value equals : \"%s\" = \"%s\"", parser_value, parser_eq); 

  /* Set the variable */
  do_set(parser_value, parser_eq);
}

include\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_include != NULL)
  {
        FREE(parser_include);
  }
  parser_include = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_include");

  strncpy( parser_include, parser_pos1, parser_pos2 - parser_pos1 );
  parser_include[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "parser.l:including file : %s", parser_include); 
  open_new_file(parser_include, yyin);

}

reverse;	{
	parser_reverse = TRUE;

}

config\ showupalso;	{
	parser_showupalso = TRUE;
}

config\ nologconnects;	{
	parser_nologconnects = TRUE;
}

config\ noheartbeat;	{
	parser_noheartbeat = TRUE;
}

config\ nosubject;	{
	parser_nosubject = TRUE;
}

config\ statusfile\ html\ [^;]+;       {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_statusfile != NULL)
  {
        FREE(parser_statusfile);
  }
  parser_statusfile = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_name");
  strncpy( parser_statusfile, parser_pos1, parser_pos2 - parser_pos1 );

  parser_statusfile[ parser_pos2 - parser_pos1 ] = '\0';
  parser_statusfile_type = 1;

  if (debug)
    print_err(0, "statusfile : %s", parser_statusfile); 

}

config\ statusfile\ text\ [^;]+;       {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_statusfile != NULL)
  {
        FREE(parser_statusfile);
  }
  parser_statusfile = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_name");
  strncpy( parser_statusfile, parser_pos1, parser_pos2 - parser_pos1 );

  parser_statusfile[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "statusfile : %s", parser_statusfile); 

}

config\ logging\ file\ [^;]+;       {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_logging != NULL)
  {
        FREE(parser_logging);
  }
  parser_logging = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_name");
  strncpy( parser_logging, parser_pos1, parser_pos2 - parser_pos1 );

  parser_logging[ parser_pos2 - parser_pos1 ] = '\0';
  if (debug)
    print_err(0, "logging : %s", parser_logging); 

  parser_logging_fac = -3;
  if (not_started_yet == TRUE)
	use_logging_now();
}

config\ logging\ syslog\ [^;]+;       {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_logging != NULL)
  {
        FREE(parser_logging);
  }
  parser_logging = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_logging");
  strncpy( parser_logging, parser_pos1, parser_pos2 - parser_pos1 );

  parser_logging[ parser_pos2 - parser_pos1 ] = '\0';
  if (debug)
    print_err(0, "logging : %s", parser_logging); 

  parser_logging_fac = match_facility(parser_logging);
  if (parser_logging_fac == -1)
  {
	print_err(0, "invalid facility %s specified", parser_logging);
	FREE(parser_logging);
	parser_logging = NULL;
  }
  if (not_started_yet == TRUE)
	use_logging_now();
}

root\ [^;]+;       {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_root != NULL)
  {
        FREE(parser_root);
  }
  parser_root = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_name");
  strncpy( parser_root, parser_pos1, parser_pos2 - parser_pos1 );

  parser_root[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "root : %s", parser_root); 

}

object[^\{]+{WS}+\{    {

  parser_reverse = FALSE;
  for (temp_int=6;strlen(yytext)> temp_int;temp_int++)
  {
	if (!isspace(yytext[temp_int]))
		break;
  }
  parser_pos1 = yytext+temp_int;
  for (;strlen(yytext)> temp_int;temp_int++)
  {
	if (isspace(yytext[temp_int]))
		break;
  }
  parser_pos2=yytext+temp_int;
 

  if (parser_name != NULL)
  {
	FREE(parser_name);
  }
  parser_name = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_name");
  strncpy( parser_name, parser_pos1, parser_pos2 - parser_pos1 );
  parser_name[parser_pos2 - parser_pos1] = '\0';

  if (debug)
    print_err(0, "object : %s {", parser_name); 


}

ip\ [^;]+;            {


  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_ip != NULL)
  {
        FREE(parser_ip);
  }
  parser_ip = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_ip");

  strncpy( parser_ip, parser_pos1, parser_pos2 - parser_pos1 );
  parser_ip[ parser_pos2 - parser_pos1 ] = '\0';

  /* Check IP to verify it is valid */

  if (debug)  
    print_err(0, "ip : %s", parser_ip); 


};


type\ [^;]+;          {

  parser_pos1 = index( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ';' );

  if (parser_type != NULL)
  {
        FREE(parser_type);
	parser_i_type = -1;
  }
  parser_type = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_type");

  strncpy( parser_type, parser_pos1, parser_pos2 - parser_pos1 );
  parser_type[ parser_pos2 - parser_pos1 ] = '\0';

  parser_i_type = name_to_type(parser_type);
  if (parser_i_type == -1)
  {
	print_err(1, "Invalid type specified in FILENAME_HERE line %d", 
		yylineno);
  }
  if (debug)
    print_err(0, "type : %s/%d ", parser_type, parser_i_type); 

};

config\ queuetime\ [^;]+;          {

  parser_pos1 = rindex( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ';' );

  if (parser_queuetime != NULL)
  {
        FREE(parser_queuetime);
	parser_i_queuetime = -1;
  }
  parser_queuetime = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_queuetime");

  strncpy( parser_queuetime, parser_pos1, parser_pos2 - parser_pos1 );
  parser_queuetime[ parser_pos2 - parser_pos1 ] = '\0';

  parser_i_queuetime = atoi(parser_queuetime);
  if (parser_i_queuetime == -1)
  {
	print_err(1, "Invalid queuetime specified in FILENAME_HERE line %d", 
		yylineno);
  }
  if (debug)
    print_err(0, "queuetime : %s/%d ", parser_queuetime, parser_i_queuetime); 

};


config\ dnsexpire\ [^;]+;          {

  parser_pos1 = rindex( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ';' );

  if (parser_dnsexpire != NULL)
  {
        FREE(parser_dnsexpire);
	parser_i_dnsexpire = -1;
  }
  parser_dnsexpire = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_dnsexpire");

  strncpy( parser_dnsexpire, parser_pos1, parser_pos2 - parser_pos1 );
  parser_dnsexpire[ parser_pos2 - parser_pos1 ] = '\0';

  parser_i_dnsexpire = atoi(parser_dnsexpire);
  if (parser_i_dnsexpire == -1)
  {
	print_err(1, "Invalid dnsexpire specified in FILENAME_HERE line %d", 
		yylineno);
  }
  if (debug)
    print_err(0, "dnsexpire : %s/%d ", parser_dnsexpire, parser_i_dnsexpire); 

};


config\ dnslog\ [^;]+;          {

  parser_pos1 = rindex( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ';' );

  if (parser_dnslog != NULL)
  {
        FREE(parser_dnslog);
	parser_i_dnslog = -1;
  }
  parser_dnslog = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_dnslog");

  strncpy( parser_dnslog, parser_pos1, parser_pos2 - parser_pos1 );
  parser_dnslog[ parser_pos2 - parser_pos1 ] = '\0';

  parser_i_dnslog = atoi(parser_dnslog);
  if (parser_i_dnslog == -1)
  {
	print_err(1, "Invalid dnslog specified in FILENAME_HERE line %d", 
		yylineno);
  }
  if (debug)
    print_err(0, "dnslog : %s/%d ", parser_dnslog, parser_i_dnslog); 

};



config\ pageinterval\ [^;]+;          {

  parser_pos1 = rindex( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ';' );

  if (parser_pageinterval != NULL)
  {
        FREE(parser_pageinterval);
	parser_i_pageinterval = -1;
  }
  parser_pageinterval = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_pageinterval");

  strncpy( parser_pageinterval, parser_pos1, parser_pos2 - parser_pos1 );
  parser_pageinterval[ parser_pos2 - parser_pos1 ] = '\0';

  parser_i_pageinterval = atoi(parser_pageinterval);
  if (parser_i_pageinterval == -1)
  {
	print_err(1, "Invalid pageinterval specified in FILENAME_HERE line %d", 
		yylineno);
  }
  if (debug)
    print_err(0, "pageinterval : %s/%d ", parser_pageinterval, parser_i_pageinterval); 

};

config\ maxqueued\ [^;]+;          {

  parser_pos1 = rindex( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ';' );

  if (parser_maxqueued != NULL)
  {
        FREE(parser_maxqueued);
	parser_i_maxqueued = -1;
  }
  parser_maxqueued = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_maxqueued");

  strncpy( parser_maxqueued, parser_pos1, parser_pos2 - parser_pos1 );
  parser_maxqueued[ parser_pos2 - parser_pos1 ] = '\0';

  parser_i_maxqueued = atoi(parser_maxqueued);
  if (parser_i_maxqueued == -1)
  {
	print_err(1, "Invalid maxqueued specified in FILENAME_HERE line %d", 
		yylineno);
  }
  if (debug)
    print_err(0, "maxqueued : %s/%d ", parser_maxqueued, parser_i_maxqueued); 

};



config\ numfailures\ [^;]+;          {

  parser_pos1 = rindex( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ';' );

  if (parser_numfailures != NULL)
  {
        FREE(parser_numfailures);
	parser_i_numfailures = -1;
  }
  parser_numfailures = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_numfailures");

  strncpy( parser_numfailures, parser_pos1, parser_pos2 - parser_pos1 );
  parser_numfailures[ parser_pos2 - parser_pos1 ] = '\0';

  parser_i_numfailures = atoi(parser_numfailures);
  if (parser_i_numfailures == -1)
  {
	print_err(1, "Invalid numfailures specified in FILENAME_HERE line %d", 
		yylineno);
  }
  if (debug)
    print_err(0, "numfailures : %s/%d ", parser_numfailures, parser_i_numfailures); 

};

config\ pmesg\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_pmesg != NULL)
  {
        FREE(parser_pmesg);
  }
  parser_pmesg = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_pmesg");

  strncpy( parser_pmesg, parser_pos1, parser_pos2 - parser_pos1 );
  parser_pmesg[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "pmesg : %s", parser_pmesg); 

};

config\ (from|sender)\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_sender != NULL)
  {
        FREE(parser_sender);
  }
  parser_sender = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_sender");

  strncpy( parser_sender, parser_pos1, parser_pos2 - parser_pos1 );
  parser_sender[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "sender : %s", parser_sender); 

};

config\ subject\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_subject != NULL)
  {
        FREE(parser_subject);
  }
  parser_subject = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_subject");

  strncpy( parser_subject, parser_pos1, parser_pos2 - parser_pos1 );
  parser_subject[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "subject : %s", parser_subject); 

};

config\ upcolor\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_upcolor != NULL)
  {
        FREE(parser_upcolor);
  }
  parser_upcolor = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_upcolor");

  strncpy( parser_upcolor, parser_pos1, parser_pos2 - parser_pos1 );
  parser_upcolor[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "upcolor : %s", parser_upcolor); 

};


config\ downcolor\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_downcolor != NULL)
  {
        FREE(parser_downcolor);
  }
  parser_downcolor = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_downcolor");

  strncpy( parser_downcolor, parser_pos1, parser_pos2 - parser_pos1 );
  parser_downcolor[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "downcolor : %s", parser_downcolor); 

};


config\ recentcolor\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_recentcolor != NULL)
  {
        FREE(parser_recentcolor);
  }
  parser_recentcolor = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_recentcolor");

  strncpy( parser_recentcolor, parser_pos1, parser_pos2 - parser_pos1 );
  parser_recentcolor[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "recentcolor : %s", parser_recentcolor); 

}

config\ replyto\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_replyto != NULL)
  {
        FREE(parser_replyto);
  }
  parser_replyto = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_replyto");

  strncpy( parser_replyto, parser_pos1, parser_pos2 - parser_pos1 );
  parser_replyto[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "replyto : %s", parser_replyto); 

};

config\ errorsto\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_errorsto != NULL)
  {
        FREE(parser_errorsto);
  }
  parser_errorsto = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_errorsto");

  strncpy( parser_errorsto, parser_pos1, parser_pos2 - parser_pos1 );
  parser_errorsto[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "errorsto : %s", parser_errorsto); 

};

config\ header\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_header != NULL)
  {
        FREE(parser_header);
  }
  parser_header = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_header");

  strncpy( parser_header, parser_pos1, parser_pos2 - parser_pos1 );
  parser_header[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "header : %s", parser_header); 

};

config\ authkey\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_authkey != NULL)
  {
        FREE(parser_authkey);
  }
  parser_authkey = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_authkey");

  strncpy( parser_authkey, parser_pos1, parser_pos2 - parser_pos1 );
  parser_authkey[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "authkey : %s", parser_authkey); 

};

port\ [^;]+;          {

  parser_pos1 = index( yytext, ' ' ) + 1;
  parser_pos2 = index( parser_pos1, ';' );

  if (parser_port != NULL)
  {
        FREE(parser_port);
	parser_i_port = -1;
  }
  parser_port = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_port");

  strncpy( parser_port, parser_pos1, parser_pos2 - parser_pos1 );
  parser_port[ parser_pos2 - parser_pos1 ] = '\0';

  parser_i_port = atoi(parser_port);
  if (parser_i_port == -1)
  {
	print_err(1, "Invalid port specified in FILENAME_HERE line %d", 
		yylineno);
  }
  if (debug)
    print_err(0, "port : %s/%d ", parser_port, parser_i_port); 

}

desc\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_desc != NULL)
  {
        FREE(parser_desc);
  }
  parser_desc = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_desc");

  strncpy( parser_desc, parser_pos1, parser_pos2 - parser_pos1 );
  parser_desc[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "desc : %s", parser_desc); 

}

secret\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_secret != NULL)
  {
        FREE(parser_secret);
  }
  parser_secret = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_secret");

  strncpy( parser_secret, parser_pos1, parser_pos2 - parser_pos1 );
  parser_secret[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "radius-secret: %s", parser_secret); 

}

url\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_url != NULL)
  {
        FREE(parser_url);
  }
  parser_url = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_url");

  strncpy( parser_url, parser_pos1, parser_pos2 - parser_pos1 );
  parser_url[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "url : %s", parser_url); 

}

urltext\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_urltext != NULL)
  {
        FREE(parser_urltext);
  }
  parser_urltext = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_urltext");

  strncpy( parser_urltext, parser_pos1, parser_pos2 - parser_pos1 );
  parser_urltext[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "urltext : %s", parser_urltext); 

}

password\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_password != NULL)
  {
        FREE(parser_password);
  }
  parser_password = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_password");

  strncpy( parser_password, parser_pos1, parser_pos2 - parser_pos1 );
  parser_password[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "password : %s", parser_password); 

}

username\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_username != NULL)
  {
        FREE(parser_username);
  }
  parser_username = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_username");

  strncpy( parser_username, parser_pos1, parser_pos2 - parser_pos1 );
  parser_username[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "username : %s", parser_username); 

}

spawn\ [^;]+;         {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_spawn != NULL)
  {
        FREE(parser_spawn);
  }
  parser_spawn = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_spawn");

  strncpy( parser_spawn, parser_pos1, parser_pos2 - parser_pos1 );
  parser_spawn[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "spawn : %s", parser_spawn); 

}


contact\ [^;]+;       {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_contact != NULL)
  {
        FREE(parser_contact);
  }
  parser_contact = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_name");
  strncpy( parser_contact, parser_pos1, parser_pos2 - parser_pos1 );

  parser_contact[ parser_pos2 - parser_pos1 ] = '\0';
  temp_char = do_set_replace(parser_contact);
  if (temp_char != NULL)
  {
	FREE(parser_contact);
	parser_contact = temp_char;
  }

  if (debug)
    print_err(0, "contact : %s", parser_contact); 

}


child\ [^;]+;         {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_child != NULL)
  {
        FREE(parser_child);
  }
  parser_child = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_name");

  strncpy( parser_child, parser_pos1, parser_pos2 - parser_pos1 );
  parser_child[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "child : %s", parser_child); 

}


dep\ [^;]+;           {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  parser_dep_tmp = MALLOC(sizeof(struct nei_list), "parser.l:parser_dep");
  parser_dep_tmp->nei_name = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_dep");

  strncpy( parser_dep_tmp->nei_name, parser_pos1, parser_pos2 - parser_pos1 );
  parser_dep_tmp->nei_name[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "dep : %s", parser_dep_tmp->nei_name); 

  parser_dep_tmp->next = parser_dep;
  parser_dep = parser_dep_tmp;

}


page\ [^;]+;          {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_page != NULL)
  {
        FREE(parser_page);
  }
  parser_page = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_page");

  strncpy( parser_page, parser_pos1, parser_pos2 - parser_pos1 );
  parser_page[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "page : %s", parser_page); 

}


also-notify\ [^;]+;   {

  parser_pos1 = index( yytext, '"' ) + 1;
  parser_pos2 = index( parser_pos1, '"' );
  if (parser_also != NULL)
  {
        FREE(parser_also);
  }
  parser_also = MALLOC((parser_pos2-parser_pos1)+1, "parse.l:parser_name");

  strncpy( parser_also, parser_pos1, parser_pos2 - parser_pos1 );
  parser_also[ parser_pos2 - parser_pos1 ] = '\0';

  if (debug)
    print_err(0, "also-notify : %s", parser_also); 

}


\};                   {

  if (debug)
    print_err(0, "close object: %s", yytext);
  add_object();

}

.		{
	print_err(1, "Unknown information found in line %d", yylineno);
}


%%

void open_new_file(char *filename, FILE *current_file)
{
	FILE *local_save;
	FILE *new_file;
	YY_BUFFER_STATE include_stack;
	
	local_save = current_file;
	include_stack = YY_CURRENT_BUFFER;

	new_file = fopen(filename, "r");
	if (new_file == NULL)
	{
                perror(filename);
                return;
	}
	yyin = new_file;
	yy_switch_to_buffer(yy_create_buffer( yyin, YY_BUF_SIZE ) );
/* 	yyrestart(new_file); */
	yylex();
	fclose(new_file);
	yy_delete_buffer( YY_CURRENT_BUFFER );

	yyin = local_save;
	yy_switch_to_buffer(include_stack);

	return;
}

void add_object()
{
	int valid_obj = 1;
	struct all_elements_list *new_ele = NULL;
	struct my_hostent *hp = NULL;

	if (parser_name == NULL)
	{
		print_err(1, "ending object that was never begun? %d ", 
			yylineno);
		valid_obj = 0;
		return;
	}

	if (parser_ip == NULL)
	{
		print_err(1, "no ip/dns name specified for object %s", 
			parser_name);
		valid_obj = 0;
	}
	hp = my_gethostbyname(parser_ip);
	if (hp == NULL)
	{
		print_err(1, "%s is not a valid dns name, not monitoring", parser_ip);
		valid_obj = 0;
	}

	if (parser_i_type == -1)
	{
		print_err(1, "unable to determine check type for object %s", 
			parser_name);
		valid_obj = 0;
	}

	if (parser_i_type == SYSM_TYPE_WWW)
	{
		if (parser_url == NULL || parser_urltext == NULL)
		{
			print_err(1, "No url or URLText specified for http check of %s object", parser_name);
			valid_obj=0;
		}
		/* Set default port, unless specified */
		if (parser_port == NULL)
		{
			parser_i_port = 80;
		}
	}
	if ( (parser_i_type == SYSM_TYPE_IMAP) || 
		(parser_i_type == SYSM_TYPE_POP3) ||
		(parser_i_type == SYSM_TYPE_POP2) || 
		(parser_i_type == SYSM_TYPE_RADIUS) )
	{
		if (parser_username == NULL)
		{
			print_err(1, "no username specified for object %s\n",
				parser_name);
			valid_obj = 0;
		}
		if (parser_password == NULL)
		{
			print_err(1, "no password specified for object %s\n",
				parser_name);
			valid_obj = 0;
		}
		if (parser_secret == NULL && 
			parser_i_type == SYSM_TYPE_RADIUS)
		{
			print_err(1, "no radius secret specified for object %s\n",
				parser_name);
			valid_obj = 0;
		}
	}

	/* should we require a description?  if so, uncomment this:
	if (parser_desc == NULL)
	{
		print_err(1, "no description specified for object %s", 
			parser_name);
		valid_obj = 0;
	}
	 */

	/* not required arguments:
		parser_spawn, parser_contact, parser_dep, parser_page, 
		parser_also 
	 */

	if (valid_obj)
	{
		if (debug)
		{
		  print_err(0, "valid object, so ADDING OBJECT: %s", 
		    parser_name);
		}
		new_ele = MALLOC(sizeof(struct all_elements_list), "parser.l:new_ele");
		new_ele->next = NULL;
		new_ele->value = MALLOC(sizeof(struct graph_elements), "parser.l:new_ele->value");

		/* NULL this out, otherwise we will FREE it above */
		new_ele->value->unique_name = parser_name;
		new_ele->value->tot_nei = 0;
		new_ele->value->num_dep = 0;
		parser_name = NULL;

		new_ele->value->data = MALLOC(sizeof(struct hostinfo), "parser.l:new_ele->value->data");

	/* LOAD DATA into structure */
		new_ele->value->data->hostname = strdup(parser_ip);
		new_ele->value->data->type = parser_i_type;
		if (parser_i_type == SYSM_TYPE_PING)
		{
			new_ele->value->data->port = 0;
		} else {
			new_ele->value->data->port = parser_i_port;
		}

		new_ele->value->data->reverse = parser_reverse;
		new_ele->value->data->contact_when_up = TRUE; /* BUG - should
			be configurable */
		new_ele->value->data->unique_id = gen_unique_id();
		new_ele->value->data->message = strdup(parser_desc);
		new_ele->value->data->snmp_community = NULL;
		new_ele->value->data->hdrval = NULL;
		new_ele->value->data->hdr = NULL;
		new_ele->value->data->lastmsgid = NULL;
		new_ele->value->data->lastcheck = 0;
		new_ele->value->data->totalchecked = 0;
		new_ele->value->data->downct = 0;
		new_ele->value->data->contacted = FALSE;
		new_ele->value->data->queued = 0;
		new_ele->value->data->lastcontacted = 0;
		new_ele->value->data->lchecktime = 0;
		new_ele->value->data->check_start = 0;
		new_ele->value->data->deathtime = 0;
		new_ele->value->data->last_up = 0;

		if (parser_contact != NULL)
		{
			new_ele->value->data->contact = strdup(parser_contact);
		} else {
			new_ele->value->data->contact = NULL;
		}
		if (parser_secret != NULL)
		{
			new_ele->value->data->secret = strdup(parser_secret);
		} else {
			new_ele->value->data->secret = NULL;
		}
		if (parser_numfailures != NULL)
		{
			new_ele->value->data->max_down = parser_i_numfailures ;
		} else {
			new_ele->value->data->max_down = numfailures;
		}
		if (parser_username != NULL)
		{
			new_ele->value->data->username = strdup(parser_username);
		} else {
			new_ele->value->data->username = NULL;
		}
		if (parser_password != NULL)
		{
			new_ele->value->data->password = strdup(parser_password);
		} else {
			new_ele->value->data->password = NULL;
		}
		if (parser_url != NULL)
		{
			new_ele->value->data->url = strdup(parser_url);
		} else {
			new_ele->value->data->url = NULL;
		}
		if (parser_urltext != NULL)
		{
			new_ele->value->data->url_text = strdup(parser_urltext);
		} else {
			new_ele->value->data->url_text = NULL;
		}
		if (parser_spawn != NULL)
		{
			new_ele->value->data->command = strdup(parser_spawn);
		} else {
			new_ele->value->data->command = NULL;
		}
	/* Done Loading data */

		new_ele->value->num_dep = 0;
		for (parser_dep_tmp = parser_dep;
			parser_dep_tmp != NULL;
			parser_dep_tmp = parser_dep_tmp->next )
		{
			new_ele->value->num_dep++;
		}

		temp_int = 0;
		if (new_ele->value->num_dep)
		{
			new_ele->value->dep_txt_name = MALLOC(sizeof(char *) * new_ele->value->num_dep, "nei_txt_name[]");

                	for (parser_dep_tmp = parser_dep;
	                        parser_dep_tmp != NULL;
	                        parser_dep_tmp = parser_dep_tmp->next )
	                {
	                        new_ele->value->dep_txt_name[temp_int] = strdup(parser_dep_tmp->nei_name);
				temp_int++;
	                }
		}

		new_ele->value->neighbors = NULL;

		/* Free parser_dep linked list */
		free_struct_nei_list(parser_dep);

		parser_dep = NULL;
		parser_dep_tmp = NULL;

		new_ele->value->visit = FALSE;

		/* Perform an insert-new-at-top-addition */
		if (parser_head == NULL)
		{
			parser_head = new_ele;
		} else {
			new_ele->next = parser_head;
			parser_head = new_ele;
		}
	} else {
		print_err(1, "did not add object %s as it was not valid", parser_name);
	}

}

void free_struct_nei_list(struct nei_list *freethis)
{
	struct nei_list *last = NULL;
	struct nei_list *here = NULL;
	
	here = freethis;
	
	while (here != NULL)
	{
		if (last != NULL)
		{
			FREE(last);
			last = NULL;
		}
		last = here;
		here=here->next;
	}
	if (last != NULL)
		FREE(last);
}

void initalize_parser()
{
	if (parser_name != NULL)
		FREE(parser_name);
	if (parser_pmesg != NULL)
		FREE(parser_pmesg);
	if (parser_ip != NULL)
		FREE(parser_ip);
	if (parser_root != NULL)
		FREE(parser_root);
	if (parser_type != NULL)
		FREE(parser_type);
	if (parser_port != NULL)
		FREE(parser_port);
	if (parser_numfailures != NULL)
		FREE(parser_numfailures);
	if (parser_desc != NULL)
		FREE(parser_desc);
	if (parser_spawn != NULL)
		FREE(parser_spawn);
	if (parser_contact != NULL)
		FREE(parser_contact);
	if (parser_child != NULL)
		FREE(parser_child);
	if (parser_sender != NULL)
		FREE(parser_sender);
	if (parser_subject != NULL)
		FREE(parser_subject);
	if (parser_upcolor != NULL)
		FREE(parser_upcolor);
	if (parser_downcolor != NULL)
		FREE(parser_downcolor);
	if (parser_recentcolor != NULL)
		FREE(parser_recentcolor);
	if (parser_replyto != NULL)
		FREE(parser_replyto);
	if (parser_errorsto != NULL)
		FREE(parser_errorsto);
	if (parser_header != NULL)
		FREE(parser_header);
	if (parser_authkey != NULL)
		FREE(parser_authkey);
	if (parser_statusfile != NULL)
		FREE(parser_statusfile);
	if (parser_logging != NULL)
		FREE(parser_logging);
	if (parser_page != NULL)
		FREE(parser_page);
	if (parser_also != NULL)
		FREE(parser_also);
	if (parser_secret != NULL)
		FREE(parser_secret);
	if (parser_username != NULL)
		FREE(parser_username);
	if (parser_password != NULL)
		FREE(parser_password);
	if (parser_url != NULL)
		FREE(parser_url);
	if (parser_urltext != NULL)
		FREE(parser_urltext);
	if (parser_include != NULL)
		FREE(parser_include);
	if (parser_queuetime != NULL)
		FREE(parser_queuetime);
	if (parser_dnsexpire != NULL)
		FREE(parser_dnsexpire);
	if (parser_dnslog != NULL)
		FREE(parser_dnslog);
	if (parser_pageinterval != NULL)
		FREE(parser_pageinterval);
	if (parser_maxqueued != NULL)
		FREE(parser_maxqueued);
	if (parser_eq != NULL)
		FREE(parser_eq);
	if (parser_value != NULL)
		FREE(parser_value);
	
/* null stuff out */
	parser_value = NULL;
	parser_eq = NULL;
	parser_name = NULL;
	parser_pmesg = NULL;
	parser_ip = NULL;
	parser_root = NULL;
	parser_type = NULL;
	parser_port = NULL;
	parser_numfailures = NULL;
	parser_desc = NULL;
	parser_spawn = NULL;
	parser_contact = NULL;
	parser_child = NULL;
	parser_sender = NULL;
	parser_subject = NULL;
	parser_upcolor = NULL;
	parser_downcolor = NULL;
	parser_recentcolor = NULL;
	parser_replyto = NULL;
	parser_errorsto = NULL;
	parser_header = NULL;
	parser_authkey = NULL;
	parser_statusfile = NULL;
	parser_logging = NULL;
	parser_page = NULL;
	parser_also = NULL;
	parser_secret = NULL;
	parser_username = NULL;
	parser_password = NULL;
	parser_url = NULL;
	parser_urltext = NULL;
	parser_include = NULL;
	parser_queuetime = NULL;
	parser_dnsexpire = NULL;
	parser_dnslog = NULL;
	parser_pageinterval = NULL;
	parser_maxqueued = NULL;
	parser_showupalso = FALSE;
	parser_nologconnects = FALSE;
	parser_noheartbeat = FALSE;
	parser_nosubject = FALSE;
}

void cleanup_after_parser()
{
	/* Same as initalizing the parser currently */

	initalize_parser();
}


void use_logging_now()
{
	facility = parser_logging_fac;
	if (facility == -3)
	{
		if (log_file != NULL)
			FREE(log_file);
		log_file = strdup(parser_logging);
	}
}

